@@ -1,9 +1,13 @@
-import itertools
 import functools
+import itertools
+
+import logbook
+
+from .._compat import itervalues, OrderedDict
 from ..ctx import context
-from .._compat import itervalues
-from .._compat import OrderedDict
-from ..utils.error_object import Error
+from .error import Error
+
+_logger = logbook.Logger(__name__)
 
 class Result(object):
 
@@ -13,9 +17,16 @@ class Result(object):
         self._errors = []
         self._failures = []
         self._skips = []
+        self._started = False
         self._finished = False
         self._interrupted = False
 
+    def is_started(self):
+        return self._started
+
+    def mark_started(self):
+        self._started = True
+
     def is_error(self):
         return bool(self._errors)
 
@@ -54,14 +65,22 @@ class Result(object):
     def is_interrupted(self):
         return self._interrupted
 
-    def add_error(self, msg=None):
-        self._add_error(self._errors, msg)
+    def add_error(self, e=None):
+        self._add_error(self._errors, e)
 
-    def add_failure(self, msg=None):
-        self._add_error(self._failures, msg)
+    def add_failure(self, e=None):
+        self._add_error(self._failures, e)
 
-    def _add_error(self, error_list, msg):
-        error_list.append(Error(msg))
+    def _add_error(self, error_list, error=None):
+        try:
+            if error is None:
+                error = Error.capture_exception()
+            if not isinstance(error, Error):
+                error = Error(error)
+            error_list.append(error)
+        except Exception:
+            _logger.error("Failed to add error to result", exc_info=True)
+            raise
 
     def add_skip(self, reason):
         self._skips.append(reason)
@@ -95,12 +114,23 @@ class GlobalResult(Result):
 
 class SessionResults(object):
 
-    def __init__(self):
+    def __init__(self, session):
         super(SessionResults, self).__init__()
+        self.session = session
         self.global_result = GlobalResult()
         self._results_dict = OrderedDict()
         self._iterator = functools.partial(itervalues, self._results_dict)
 
+    def iter_all_failures(self):
+        for result in self.iter_all_results():
+            if result.get_failures():
+                yield result, result.get_failures()
+
+    def iter_all_errors(self):
+        for result in self.iter_all_results():
+            if result.get_errors():
+                yield result, result.get_errors()
+
     @property
     def current(self):
         test_id = context.test_id
@@ -119,6 +149,9 @@ class SessionResults(object):
     def get_num_results(self):
         return len(self._results_dict)
 
+    def get_num_started(self):
+        return self._count(Result.is_started, include_global=False)
+
     def get_num_successful(self):
         return self._count(Result.is_success_finished, include_global=False)
 
